
# Disseny i implementació d'un sistema de sensors intel·ligent per a la supervisió de pacients en habitacions de centres geriàtrics

Aquest projecte implementa un sistema complet per a la supervisió de pacients en habitacions de geriàtrics mitjançant sensors intel·ligents. Inclou detecció de caigudes, monitorització de temperatura i humitat, i una interfície gràfica d'usuari per supervisar en temps real.

---

## Estructura del Projecte

El projecte està dividit en tres carpetes principals:

### 1. `TFG_Arduino`
Conté el codi que s'executa als microcontroladors Arduino.

- **`TFG_caigudes`**: Detecta caigudes utilitzant models entrenats a Edge Impulse.
- **`TFG_temp_humitat`**: Llegeix i envia dades de temperatura i humitat des del sensor HTS221.
- **`TFG_caigudes_inferencing`**: Llibreria necessària per a la detecció de caigudes que s'ha de descarregar i instal·lar al projecte.

### 2. `TFG_raspberry`
Conté el codi que s'executa a la Raspberry Pi, que rep dades dels Arduino i les publica al broker MQTT.

- **Funcions principals**:
  - Recull dades de temperatura i humitat d'un port sèrie.
  - Detecta caigudes mitjançant un altre port sèrie.
  - Publica dades al broker MQTT per a la supervisió remota.

### 3. `TFG_GUI`
Conté la interfície gràfica d'usuari (GUI) per monitoritzar l'estat de les habitacions en temps real.

---

## Configuració

### 1. Arduino
- **Càrrega del codi**: Carrega els programes corresponents als microcontroladors.
  - `TFG_caigudes` per al dispositiu encarregat de detectar caigudes.
  - `TFG_temp_humitat` per al dispositiu que mesura temperatura i humitat.
- **Dependències**: Assegura't de tenir instal·lades les biblioteques necessàries com `Arduino_LSM9DS1` i `Arduino_HTS221`.
- **Model Edge Impulse**: Assegura't que la llibreria inferencing (`TFG_caigudes_inferencing`) està instal·lada correctament.

### 2. Raspberry Pi
- **IP del Broker MQTT**: Configura la IP del broker MQTT i el port a les línies corresponents del codi.
- **Ports sèrie**: Actualitza les rutes per als dispositius connectats.

### 3. GUI

- Les instruccions específiques de la GUI es troben en un readme.md a part

### 4. Broker MQTT
- Es requereix un equip (pot ser l'ordinador on s'execute la GUI o un altre dispositiu de la xarxa) que actue com a broker MQTT. Es recomana usar **Mosquitto**.
- Per instal·lar Mosquitto:
  ```bash
  sudo apt update
  sudo apt install mosquitto mosquitto-clients
  ```
- Configura el broker segons les necessitats i anota la seva IP per introduir-la als codis de la Raspberry Pi i de la GUI.

---

## Execució

### 1. Arduino
- Carrega el codi corresponent als microcontroladors utilitzant l'entorn Arduino IDE.

### 2. Raspberry Pi
- Executa el programa de la Raspberry Pi:
  ```bash
  python3 TFG_Raspberry.py
  ```

### 3. GUI
#### Windows
- Executa el programa amb:
  ```bash
  python main.py
  ```

#### Linux
- Assegura't que tens les dependències instal·lades i executa:
  ```bash
  python3 main.py
  ```

---

## Requisits

- **Arduino**:
  - 2x Arduino Nano 33 BLE Sense.

- **Raspberry Pi**:
  - Raspberry Pi amb Python 3.6 o superior.

- **GUI**:
  - Python 3.6 o superior.
  - Llibreries:
    - `PyQt5`
    - `paho-mqtt`

## Autor

- Nom: Jordi Capó Calabuig
- Email: jordicapocalabuig@gmail.com
- TFG dirigit per: Antonio Molina Picó

## Demostració de funcionament

https://www.youtube.com/watch?v=8VkPqJJAkV8
