/*
 * Copyright (c) 2024 EdgeImpulse Inc.
 *
 * Generated by Edge Impulse and licensed under the applicable Edge Impulse
 * Terms of Service. Community and Professional Terms of Service
 * (https://docs.edgeimpulse.com/page/terms-of-service) or Enterprise Terms of
 * Service (https://docs.edgeimpulse.com/page/enterprise-terms-of-service),
 * according to your product plan subscription (the “License”).
 *
 * This software, documentation and other associated files (collectively referred
 * to as the “Software”) is a single SDK variation generated by the Edge Impulse
 * platform and requires an active paid Edge Impulse subscription to use this
 * Software for any purpose.
 *
 * You may NOT use this Software unless you have an active Edge Impulse subscription
 * that meets the eligibility requirements for the applicable License, subject to
 * your full and continued compliance with the terms and conditions of the License,
 * including without limitation any usage restrictions under the applicable License.
 *
 * If you do not have an active Edge Impulse product plan subscription, or if use
 * of this Software exceeds the usage limitations of your Edge Impulse product plan
 * subscription, you are not permitted to use this Software and must immediately
 * delete and erase all copies of this Software within your control or possession.
 * Edge Impulse reserves all rights and remedies available to enforce its rights.
 *
 * Unless required by applicable law or agreed to in writing, the Software is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing
 * permissions, disclaimers and limitations under the License.
 */

#ifndef _EI_CLASSIFIER_MODEL_VARIABLES_H_
#define _EI_CLASSIFIER_MODEL_VARIABLES_H_

#include <stdint.h>
#include "model_metadata.h"
#include "anomaly_metadata.h"
#include "tflite-model/tflite_learn_11.h"

#include "edge-impulse-sdk/classifier/ei_model_types.h"
#include "edge-impulse-sdk/classifier/inferencing_engines/engines.h"

const char* ei_classifier_inferencing_categories[] = { "passos", "res" };

uint8_t ei_dsp_config_10_axes[] = { 0, 1, 2 };
const uint32_t ei_dsp_config_10_axes_size = 3;
ei_dsp_config_spectral_analysis_t ei_dsp_config_10 = {
    10, // uint32_t blockId
    4, // int implementationVersion
    3, // int length of axes
    1.0f, // float scale-axes
    1, // int input-decimation-ratio
    "none", // select filter-type
    3.0f, // float filter-cutoff
    6, // int filter-order
    "FFT", // select analysis-type
    16, // int fft-length
    3, // int spectral-peaks-count
    0.1f, // float spectral-peaks-threshold
    "0.1, 0.5, 1.0, 2.0, 5.0", // string spectral-power-edges
    true, // boolean do-log
    true, // boolean do-fft-overlap
    1, // int wavelet-level
    "db8", // select wavelet
    false // boolean extra-low-freq
};

const uint8_t ei_dsp_blocks_size = 1;
ei_model_dsp_t ei_dsp_blocks[ei_dsp_blocks_size] = {
    { // DSP block 10
        10,
        39, // output size
        &extract_spectral_analysis_features, // DSP function pointer
        (void*)&ei_dsp_config_10, // pointer to config struct
        ei_dsp_config_10_axes, // array of offsets into the input stream, one for each axis
        ei_dsp_config_10_axes_size, // number of axes
        1, // version
        nullptr, // factory function
    }
};
const ei_config_tflite_graph_t ei_config_tflite_graph_11 = {
    .implementation_version = 1,
    .model = tflite_learn_11,
    .model_size = tflite_learn_11_len,
    .arena_size = tflite_learn_11_arena_size
};


const ei_learning_block_config_tflite_graph_t ei_learning_block_config_11 = {
    .implementation_version = 1,
    .classification_mode = EI_CLASSIFIER_CLASSIFICATION_MODE_CLASSIFICATION,
    .block_id = 11,
    .object_detection = 0,
    .object_detection_last_layer = EI_CLASSIFIER_LAST_LAYER_UNKNOWN,
    .output_data_tensor = 0,
    .output_labels_tensor = 1,
    .output_score_tensor = 2,
    .threshold = 0,
    .quantized = 1,
    .compiled = 0,
    .graph_config = (void*)&ei_config_tflite_graph_11
};

const ei_learning_block_config_anomaly_kmeans_t ei_learning_block_config_12 = {
    .implementation_version = 1,
    .classification_mode = EI_CLASSIFIER_CLASSIFICATION_MODE_ANOMALY_KMEANS,
    .anom_axis = ei_classifier_anom_axes,
    .anom_axes_size = 3,
    .anom_clusters = ei_classifier_anom_clusters,
    .anom_cluster_count = 32,
    .anom_scale = ei_classifier_anom_scale,
    .anom_mean = ei_classifier_anom_mean,
};

const uint8_t ei_learning_blocks_size = 2;
const uint32_t ei_learning_block_11_inputs[1] = { 10 };
const uint8_t ei_learning_block_11_inputs_size = 1;
const uint32_t ei_learning_block_12_inputs[1] = { 10 };
const uint8_t ei_learning_block_12_inputs_size = 1;
const ei_learning_block_t ei_learning_blocks[ei_learning_blocks_size] = {
    {
        11,
        false,
        &run_nn_inference,
        (void*)&ei_learning_block_config_11,
        EI_CLASSIFIER_IMAGE_SCALING_NONE,
        ei_learning_block_11_inputs,
        ei_learning_block_11_inputs_size,
        2
    },
    {
        12,
        false,
        &run_kmeans_anomaly,
        (void*)&ei_learning_block_config_12,
        EI_CLASSIFIER_IMAGE_SCALING_NONE,
        ei_learning_block_12_inputs,
        ei_learning_block_12_inputs_size,
        1
    },
};


const ei_object_detection_nms_config_t ei_object_detection_nms = {
    0.0f, /* NMS confidence threshold */
    0.2f  /* NMS IOU threshold */
};

const ei_impulse_t impulse_541499_0 = {
    .project_id = 541499,
    .project_owner = "Jordi Capó",
    .project_name = "TFG_caigudes",
    .impulse_id = 3,
    .impulse_name = "Impulse #3",
    .deploy_version = 4,

    .nn_input_frame_size = 39,
    .raw_sample_count = 200,
    .raw_samples_per_frame = 3,
    .dsp_input_frame_size = 200 * 3,
    .input_width = 0,
    .input_height = 0,
    .input_frames = 0,
    .interval_ms = 10,
    .frequency = 100,
    .dsp_blocks_size = ei_dsp_blocks_size,
    .dsp_blocks = ei_dsp_blocks,
    
    .object_detection_count = 0,
    .fomo_output_size = 0,

    
    .visual_ad_grid_size_x = 0,
    .visual_ad_grid_size_y = 0,
    
    .tflite_output_features_count = 2,
    .learning_blocks_size = ei_learning_blocks_size,
    .learning_blocks = ei_learning_blocks,

    .postprocessing_blocks_size = 0,
    .postprocessing_blocks = nullptr,

    .inferencing_engine = EI_CLASSIFIER_TFLITE,

    .sensor = EI_CLASSIFIER_SENSOR_ACCELEROMETER,
    .fusion_string = "accX + accY + accZ",
    .slice_size = (200/4),
    .slices_per_model_window = 4,

    .has_anomaly = EI_ANOMALY_TYPE_KMEANS,
    .label_count = 2,
    .categories = ei_classifier_inferencing_categories,
    .object_detection_nms = ei_object_detection_nms
};

ei_impulse_handle_t impulse_handle_541499_0 = ei_impulse_handle_t( &impulse_541499_0 );
ei_impulse_handle_t& ei_default_impulse = impulse_handle_541499_0;

#endif // _EI_CLASSIFIER_MODEL_METADATA_H_
