from PyQt5.QtWidgets import QWidget, QVBoxLayout, QPushButton, QSpacerItem, QSizePolicy, QFrame, QMessageBox
from PyQt5.QtCore import Qt, QThread, pyqtSignal
from PyQt5.QtMultimedia import QSound
import os
from PyQt5.QtWidgets import QGraphicsDropShadowEffect
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QPushButton, QFrame, QMessageBox
from PyQt5.QtGui import QColor
from PyQt5.QtCore import Qt, QThread, pyqtSignal
from configuracio.constants import assets_dir, BASE_ASSETS_DIR


class AlertaCaiguda(QThread):
    """
    Fil separat per gestionar notificacions d'una caiguda detectada.
    """
    alerta_signal = pyqtSignal(str)  # Senyal per emetre el missatge de caiguda

    def __init__(self, habitacio_nom):
        """
        Inicialitza el fil per gestionar alertes de caiguda.

        Paràmetres:
        ----------
        habitacio_nom (str): Nom de l'habitació associada a l'alerta.
        """
        super().__init__()
        self.habitacio_nom = habitacio_nom

    def run(self):
        """
        Executa el fil i emet el senyal amb el missatge d'alerta.
        """
        self.alerta_signal.emit(f"Possible caiguda detectada a {self.habitacio_nom}!")


class Habitacio(QWidget):
    """
    Classe que representa una habitació del geriàtric a la interfície gràfica.
    """
    def __init__(self, nom, temp_min_norm=20, temp_max_norm=30, hum_min_norm=30, hum_max_norm=60):
        """
        Inicialitza l'habitació amb els seus valors predeterminats.

        Paràmetres:
        ----------
        nom (str): Nom de l'habitació.
        temp_min_norm (int): Temperatura mínima normal permesa.
        temp_max_norm (int): Temperatura màxima normal permesa.
        hum_min_norm (int): Humitat mínima normal permesa.
        hum_max_norm (int): Humitat màxima normal permesa.
        """

        super().__init__()
        self.nom = nom
        self.temp_min_norm = temp_min_norm
        self.temp_max_norm = temp_max_norm
        self.hum_min_norm = hum_min_norm
        self.hum_max_norm = hum_max_norm

        # Valors actuals de temperatura i humitat
        self.temperatura_actual = None
        self.humitat_actual = None
        # Variable per a controlar si hi ha una caiguda detectada
        self.caiguda_detectada = False

        self.initUI()

    def initUI(self):
        """
        Configura la interfície gràfica de l'habitació, incloent el llit i els elements visuals.
        """

        layout = QVBoxLayout()

        # Quadrat que representa l'habitació
        self.habitacio_label = QPushButton(self.nom, self)
        self.habitacio_label.setFixedSize(250, 250)
        self.habitacio_label.setStyleSheet("""
            QPushButton {
                background-color: #37B47E;  
                color: white;
                border-radius: 15px;
                border: 1px solid black;
                font-family: 'Eurostile', sans-serif;  
                font-size: 18px;  
                font-weight: bold;  
            }
        """)

        # Efecte d'ombra
        ombra = QGraphicsDropShadowEffect()
        ombra.setBlurRadius(20) 
        ombra.setOffset(5, 5)    
        ombra.setColor(Qt.gray)  

        self.habitacio_label.setGraphicsEffect(ombra)

        layout.addWidget(self.habitacio_label)      
        self.setLayout(layout)
        self.habitacio_label.clicked.connect(self.mostrar_alerta)  
        
        
        # Layout per al llit
        self.llit_layout = QVBoxLayout(self.habitacio_label)
        spacer = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.llit_layout.addItem(spacer)

        self.llit_label = QPushButton("", self.habitacio_label)
        self.llit_label.setFixedSize(170, 80)
        self.llit_label.setStyleSheet("""
            QPushButton {
                background-color: #6DD8A5;  
                border-radius: 10px;
                border: 1px solid black;
            }
        """)    

        # Aplica ombra al llit
        ombra_llit = QGraphicsDropShadowEffect()
        ombra_llit.setBlurRadius(15)
        ombra_llit.setOffset(3, 3)
    

        ombra_llit.setColor(QColor(0, 100, 0))  # Verd fosc 

        self.llit_label.setGraphicsEffect(ombra_llit)

        self.llit_layout.addWidget(self.llit_label, alignment=Qt.AlignTop)

        # Connectar el clic del llit a la funció mostrar_dades
        self.llit_label.clicked.connect(self.mostrar_dades)

        # Coixí
        self.coixi_label = QFrame(self.llit_label)
        self.coixi_label.setFixedSize(30, 60)
        self.coixi_label.setStyleSheet("""
            QFrame {
                background-color: #6DD8A5;  
                border: 1px solid black;
                border-radius: 5px;
            }
        """)
        self.coixi_label.move(10, 10)

        # Ombra per al coixí
        ombra_coixi = QGraphicsDropShadowEffect()
        ombra_coixi.setBlurRadius(10)  
        ombra_coixi.setOffset(2, 2)  
        ombra_coixi.setColor(QColor(0, 100, 0))  # Verd fosc
        self.coixi_label.setGraphicsEffect(ombra_coixi)

        self.setLayout(layout)

    def mostrar_dades(self):
        """
        Mostra un diàleg amb informació sobre els valors normals i actuals de l'habitació.
        """

        # Preparar el text per mostrar els valors normals
        dades_normals = f"Temperatura normal: {self.temp_min_norm}ºC - {self.temp_max_norm}ºC\n"
        dades_normals += f"Humitat normal: {self.hum_min_norm}% - {self.hum_max_norm}%\n"

        # Preparar el text per mostrar els valors actuals, o indicar si no s'han rebut
        if self.temperatura_actual is not None:
            dades_actuals = f"Temperatura actual: {self.temperatura_actual}ºC\n"
        else:
            dades_actuals = "Temperatura actual: No disponible\n"

        if self.humitat_actual is not None:
            dades_actuals += f"Humitat actual: {self.humitat_actual}%\n\n"
        else:
            dades_actuals += "Humitat actual: No disponible\n\n"

        # Crear el QMessageBox
        msg_box = QMessageBox(self)
        msg_box.setWindowTitle(f"Dades {self.nom}")  

        # Aplicar estil a la finestra de diàleg
        msg_box.setStyleSheet("""
            QMessageBox {
                background-color: #ffffff;  
                font-family: 'Eurostile', sans-serif;  
                font-size: 16px;  
                color: #333333;  
            }
            QPushButton {
                background-color: #37B47E;  
                color: white;
                border: none;
                padding: 10px 20px;
                border-radius: 10px;
                font-size: 14px;
                font-family: 'Eurostile', sans-serif;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #2e9e6f;  
            }
            QPushButton:pressed {
                background-color: #26865f;  
            }
        """)

        # Afegir el text a la finestra de diàleg
        msg_box.setText(dades_actuals + dades_normals)

        # Mostrar el QMessageBox
        msg_box.exec_()

    def mostrar_alerta(self):
        """
        Mostra una finestra emergent d'alerta en cas que l'usuari faça clic a una habitació on s'ha detectat una caiguda.
        Permet que l'usuari confirme que l'alerta ha estat atesa.
        """

        if self.caiguda_detectada:
            # Crear un QMessageBox amb dos botons personalitzats
            msg_box = QMessageBox(self)
            msg_box.setWindowTitle(f"Alerta {self.nom}")
            msg_box.setText("Possible caiguda detectada")

            # Afegir els dos botons
            btn_atendre = msg_box.addButton("Caiguda atesa", QMessageBox.ActionRole)
            btn_tancar = msg_box.addButton("Tancar", QMessageBox.RejectRole)

            # Aplicar estil al diàleg
            msg_box.setStyleSheet("""
            QMessageBox {
                background-color: #FFFFFF;  
                font-family: 'Eurostile', sans-serif;
                font-size: 16px;
                color: #333333;
            }
            QPushButton {
                background-color: #37B47E;
                color: white;
                border-radius: 10px;
                padding: 10px;
                font-family: 'Eurostile', sans-serif;
                font-size: 14px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #2e9e6f;
            }
            QPushButton:pressed {
                background-color: #26865f;
            }
        """)
            
            # Estils per als botons
            btn_atendre.setStyleSheet("""
                QPushButton {
                    background-color: #37B47E;  
                    color: white;
                    border-radius: 10px;
                    padding: 10px;
                    font-family: 'Eurostile', sans-serif;
                    font-size: 14px;
                    font-weight: bold;
                }
                QPushButton:hover {
                    background-color: #2e9e6f;
                }
                QPushButton:pressed {
                    background-color: #26865f;
                }
            """)

            btn_tancar.setStyleSheet("""
                QPushButton {
                    background-color: #FF6347;  
                    color: white;
                    border-radius: 10px;
                    padding: 10px;
                    font-family: 'Eurostile', sans-serif;
                    font-size: 14px;
                    font-weight: bold;
                }
                QPushButton:hover {
                    background-color: #e53d2f;
                }
                QPushButton:pressed {
                    background-color: #d92f22;
                }
            """)

            # Mostrar el diàleg i esperar a vore quin botó es clica
            msg_box.exec_()

            # Si es clica el botó "Caiguda atesa"
            if msg_box.clickedButton() == btn_atendre:
                self.caiguda_detectada = False
                # Reinicialitzar el color de l'habitació
                color = "#37B47E"  # Verd original 

                # Actualitzar l'estil de l'habitació
                self.habitacio_label.setStyleSheet(f"""
                QPushButton {{
                    background-color: {color};
                    color: white;
                    border-radius: 15px;
                    border: 1px solid black;
                    font-family: 'Eurostile', sans-serif;
                    font-size: 18px;
                    font-weight: bold;
                }}
            """)

    # Funció per gestionar la caiguda detectada
    def detectar_caiguda(self):
        """
        Activa l'estat d'alerta per una caiguda detectada, canviant l'estil visual
        de l'habitació i reproduint un so d'alerta.
        """

        self.caiguda_detectada = True
        color = "#FF6347"  # Color roig

        # Assegurar que només es modifica l'habitació específica
        self.habitacio_label.setStyleSheet(f"""
        QPushButton {{
            background-color: {color};
            color: white;
            border-radius: 15px;
            border: 1px solid black;
            font-family: 'Eurostile', sans-serif;
            font-size: 18px;
            font-weight: bold;
        }}
    """)
        
        # Reproduir el so d'alerta
        sound_path = os.path.join(BASE_ASSETS_DIR, 'alarma.wav')  
        QSound.play(sound_path)

        # Crear un fil per mostrar la notificació de la caiguda
        self.alerta_caiguda = AlertaCaiguda(self.nom)
        self.alerta_caiguda.alerta_signal.connect(self.mostrar_alerta_automatica)
        self.alerta_caiguda.start()

    def mostrar_alerta_automatica(self, missatge):
        """
        Mostra una finestra emergent automàtica amb un missatge d'alerta.

        Paràmetres:
        ----------
        missatge (str): Missatge a mostrar a la finestra emergent.
        """

        # Crear un QMessageBox amb un text d'alerta
        msg_box = QMessageBox(self)
        msg_box.setWindowTitle(f"Possible caiguda a {self.nom}")
        msg_box.setText(missatge)

        
        # Personalitzar el botó "Tancar"
        btn_tancar = QPushButton("Tancar", self)
        btn_tancar.setStyleSheet("""
            QPushButton {
                background-color: #FF6347;
                color: white;
                border-radius: 10px;
                padding: 10px;
                font-family: 'Eurostile', sans-serif;
                font-size: 14px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #e53d2f;
            }
            QPushButton:pressed {
                background-color: #d92f22;
            }
        """)
        btn_tancar.clicked.connect(msg_box.close)  # Tancar el diàleg 

        # Afegir els botons personalitzats al diàleg
        msg_box.addButton(btn_tancar, QMessageBox.RejectRole)

        # Mostrar el diàleg
        msg_box.exec_()

    def on_caiguda_atesa(self):
        """
        Restableix l'estat visual de l'habitació després de gestionar una caiguda detectada.
        """

        color = "#37B47E"  # Verd original
        self.habitacio_label.setStyleSheet(f"""
            QPushButton {{
                background-color: {color};
                color: white;
                border-radius: 15px;
                border: 1px solid black;
                font-family: 'Eurostile', sans-serif;
                font-size: 18px;
                font-weight: bold;
            }}
        """)
        # restablir el color del llit i coixí
        self.llit_label.setStyleSheet(f"""
            QPushButton {{
                background-color: {color};
                border-radius: 10px;
                border: 1px solid black;
            }}
        """)
        self.coixi_label.setStyleSheet(f"""
            QFrame {{
                background-color: {color};
                border: 1px solid black;
                border-radius: 5px;
            }}
        """)

        
    # Funció per actualitzar colors 
    def actualitzar_color_llit(self, temperatura=None, humitat=None):
        """
        Actualitza el color del llit en funció dels valors de temperatura i humitat.

        Paràmetres:
        ----------
        temperatura (float, opcional): Nova temperatura detectada.
        humitat (float, opcional): Nova humitat detectada.
        """

        if temperatura is not None:
            self.temperatura_actual = temperatura
        if humitat is not None:
            self.humitat_actual = humitat

        if self.temperatura_actual is not None and self.humitat_actual is not None:
            if (self.temp_min_norm <= self.temperatura_actual <= self.temp_max_norm) and \
            (self.hum_min_norm <= self.humitat_actual <= self.hum_max_norm):
                color = "#6DD8A5"  
            else:
                color = "#FF6347" 
        else:
            if (self.temperatura_actual is not None and not (self.temp_min_norm <= self.temperatura_actual <= self.temp_max_norm)) or \
            (self.humitat_actual is not None and not (self.hum_min_norm <= self.humitat_actual <= self.hum_max_norm)):
                color = "#FF6347"  
            else:
                color = "#6DD8A5" 

    
        # Actualitzar el color del llit i coixí
        self.llit_label.setStyleSheet(f"""
            QPushButton {{
                background-color: {color};
                border-radius: 10px;
                border: 1px solid black;
            }}
        """)
        self.coixi_label.setStyleSheet(f"""
            QFrame {{
                background-color: {color};
                border: 1px solid black;
                border-radius: 5px;
            }}
        """)