import os
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QLabel, QPushButton, QSizePolicy, QSpacerItem
from PyQt5.QtGui import QPixmap
from PyQt5.QtCore import Qt
from configuracio.constants import assets_dir, BASE_ASSETS_DIR

class PantallaInici(QWidget):
    """
    Pantalla inicial del programa
    """
    def __init__(self, app_principal):
        """
        Inicialitza la pantalla d'inici.

        Paràmetres:
        ----------
        app_principal (QWidget): Instància de l'aplicació principal a la qual es redirigeix l'usuari.
        """

        super().__init__()
        self.app_principal = app_principal
        self.initUI()

    def initUI(self):
        """
        Configura la interfície gràfica de la pantalla inicial.
        """

        layout = QVBoxLayout()
        self.setStyleSheet("background-color: #ffffff;")

        layout.addSpacerItem(QSpacerItem(30, 40, QSizePolicy.Minimum, QSizePolicy.Expanding))

        # Carregar el logo
        self.logo_label = QLabel(self)
        icon_path = os.path.join(BASE_ASSETS_DIR, "icono.png")
        pixmap = QPixmap(icon_path)
        if pixmap.isNull():
            print("Error: No s'ha pogut carregar el logo.")
        pixmap = pixmap.scaled(round(pixmap.width() * 1.5), round(pixmap.height() * 1.5), Qt.KeepAspectRatio, Qt.SmoothTransformation)

        self.logo_label.setPixmap(pixmap)
        self.logo_label.setPixmap(pixmap)
        self.logo_label.setAlignment(Qt.AlignCenter)
        layout.addWidget(self.logo_label)

        # Botó "Iniciar"
        self.boto_iniciar = QPushButton("Iniciar", self)
        self.boto_iniciar.setFixedWidth(150)  

        # Aplicar estil al botó
        self.boto_iniciar.setStyleSheet("""
            QPushButton {
                background-color: #37B47E;
                color: white;
                border: none;
                padding: 10px 20px;
                border-radius: 15px;
                font-size: 16px;
                font-family: 'Eurostile', sans-serif;  
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #2e9e6f;
            }
            QPushButton:pressed {
                background-color: #26865f;
            }
        """)

        
        layout.addWidget(self.boto_iniciar, alignment=Qt.AlignCenter)  # Centrar el botó

        self.boto_iniciar.clicked.connect(self.mostrar_app_principal)

        # Espaiador inferior per centrar el contingut verticalment
        layout.addSpacerItem(QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding))

        self.setLayout(layout)
        self.setWindowTitle("Pantalla d'Inici")
        self.showMaximized()

    def mostrar_app_principal(self):
        """
        Mostra la finestra de l'aplicació principal i tanca la pantalla d'inici.
        """
        self.app_principal.showMaximized()
        self.close()