import os
import json

# Configuració
CONFIG_FILE = "config.json"

def clear_layout(layout):
    """Elimina tots els widgets d'un layout."""

    while layout.count():
        item = layout.takeAt(0)
        widget = item.widget()
        if widget is not None:
            widget.setParent(None)  

def save_config(data):
    """
    Guarda la configuració en un fitxer JSON.

    Paràmetres:
    ----------
    data (dict): Configuració que es vol guardar.
    """
    try:
        with open(CONFIG_FILE, 'w') as f:
            json.dump(data, f, indent=4)
    except Exception as e:
        print(f"Error guardant la configuració: {e}")

def load_config():
    """
    Carrega la configuració des d'un fitxer JSON o retorna una configuració predeterminada.

    Retorna:
    --------
    dict: Configuració carregada o valors predeterminats.
    """
    if os.path.exists(CONFIG_FILE):
        try:
            with open(CONFIG_FILE, 'r') as f:
                config = json.load(f)
                if "num_habitacions" not in config:
                    config["num_habitacions"] = 9  # Assigna el valor per defecte si no està definit
                return config
        except Exception as e:
            print(f"Error carregant la configuració: {e}")
    # Configuració predeterminada si no existeix el fitxer
    return {
        "habitacions": {
            f"Habitació {i+1}": {"temp_min": 20, "temp_max": 30, "hum_min": 30, "hum_max": 60}
            for i in range(9)  # Configura fins a 9 habitacions per defecte
        },
        "num_habitacions": 9
    }
