# TFG - GUI

## Descripció

Interfície gràfica d'usuari (GUI) desenvolupada en Python amb PyQt5, dissenyada per a la monitorització en temps real de pacients en habitacions de centres geriàtrics. La GUI proporciona una forma intuïtiva de supervisar els valors de temperatura, humitat i alertes per caigudes, utilitzant un sistema de sensors connectat mitjançant MQTT.

---

## Característiques

- **Monitorització en temps real:** Visualització de temperatura corporal i humitat al llit de cada habitació amb codis de colors per identificar anomalies.
- **Alertes per caigudes:** Alerta visual i sonora quan es detecta una caiguda.
- **Configuració personalitzada:** Opcions per modificar els paràmetres normals (temperatura, humitat) de cada habitació.
- **Integració amb MQTT:** Subscriu i processa dades des d'un broker MQTT.

---

## Requisits del Sistema

- Python 3.8 o superior
- **Llibreries Python necessàries:**
  - `PyQt5`
  - `paho-mqtt`
  - `pydub`

Per instal·lar les dependències:

```bash
pip install -r requirements.txt
```

---

## Estructura del Projecte

```
TFG_GUI/
│
├── assets/                # Recursos visuals i multimèdia
│   ├── icono.png          # Icona principal de la GUI
│   ├── fondo.png          # Imatge de fons de la GUI
│   └── alarma.wav         # So d'alerta per caigudes
│
├── configuracio/          # Configuració i constants      
│   ├── constants.py       # Constants del projecte
│   └── gestio_config.py   # Funcions per carregar/guardar configuració
│
├── components/            # Components principals de la GUI
│   ├── config_habitacio.py # Configuració individual d'habitacions
│   ├── habitacio.py       # Classe per gestionar cada habitació
│   ├── pantalla_inici.py  # Pantalla d'inici
│   ├── configuracio.py    # Configuració general de la GUI
│   └── aplicacio_principal.py # Interfície principal de la GUI
│
├── README.md              # Documentació del projecte
├── requirements.txt       # Dependències del projecte
└── main.py                # Punt d'entrada principal
└── main.py                # Configuració guardada
```

---

## Ús

1. **Assegura't que tots els recursos (imatges, sons) estan disponibles a la carpeta `assets/`.**
2. **Configura el broker MQTT:** Obre el fitxer `constants.py` dins de la carpeta `configuracio` i modifica les línies 17 i 18 per incloure la IP i el port correctes del broker MQTT:

```python
# configuracio/constants.py
MQTT_BROKER = "IP_DEL_BROKER"  # IP del broker MQTT
MQTT_PORT = 1883              # Port del broker MQTT
```

3. **Executa l'aplicació:**
   - Per **Windows**:
     ```bash
     python main.py
     ```
   - Per **Linux**:
     ```bash
     python3 main.py
     ```


4. Interactua amb la GUI:
   - La pantalla principal mostra l'estat de cada habitació.
   - Les anomalies de temperatura i humitat es mostren en color roig.
   - Les caigudes es notifiquen amb un so i una alerta emergent.
   - Baix a la dreta hi ha un botó d'ajuda per més instruccions.

---

## Configuració

El fitxer `config.json` guarda els valors per de la configuració sempre que es modifiquen.


## Autor

Aquest projecte ha estat desenvolupat com a part del Treball Final de Grau: Disseny i implementació d'un sistema de sensors intel·ligent per a la supervisió de pacients en habitacions de centres geriàtrics.

**Contacte:** 

jordicapocalabuig@gmail.com

**Demostració de funcionament:** 

https://www.youtube.com/watch?v=8VkPqJJAkV8


