import serial
import time
import paho.mqtt.client as mqtt

# Configuració MQTT
MQTT_BROKER = "192.168.1.89"  # IP del broker MQTT 
MQTT_PORT = 1883
TOPIC_TEMPERATURE = "geriatric/habitacio1/temperatura"
TOPIC_HUMIDITY = "geriatric/habitacio1/humitat"
TOPIC_ALERTA_CAIGUDA = "geriatric/habitacio1/alerta"

# Configuració ports sèrie
SERIAL_PORT_TEMPHUM = '/dev/ttyACM1'  # Port de temperatura i humitat
BAUD_RATE_TEMPHUM = 9600

SERIAL_PORT_CAIGUDA = '/dev/ttyACM0'  # Port per a la detecció de caigudes
BAUD_RATE_CAIGUDA = 115200

# Inicialitzar els ports sèrie
ser_temphum = serial.Serial(SERIAL_PORT_TEMPHUM, BAUD_RATE_TEMPHUM, timeout=1)
ser_caiguda = serial.Serial(SERIAL_PORT_CAIGUDA, BAUD_RATE_CAIGUDA, timeout=1)

# Funció per enviar dades MQTT
def send_mqtt_data(client, temperature, humidity):
    client.publish(TOPIC_TEMPERATURE, temperature)
    client.publish(TOPIC_HUMIDITY, humidity)

def send_mqtt_alert(client):
    client.publish(TOPIC_ALERTA_CAIGUDA, "Caiguda detectada")

# Inicialitzar el client MQTT
client = mqtt.Client()
client.connect(MQTT_BROKER, MQTT_PORT, 60)

# Control de duplicats
last_caiguda_time = 0
CAIGUDA_INTERVAL = 10

# Variable per evitar missatges duplicats de caigudes consecutives
caiguda_detectada = False

# Bucle per llegir dades dels ports sèrie i enviar-les per MQTT
try:
    while True:
        # Llegir dades de temperatura i humitat
        if ser_temphum.in_waiting > 0:
            line = ser_temphum.readline().decode('utf-8').strip()
            if line:
                print(f"Dades rebudes: {line}")
                # Dividir les dades rebudes
                parts = line.split(';')
                if len(parts) == 2:
                    temperatura = parts[0].split(':')[1]
                    humitat = parts[1].split(':')[1]
                    print(f"Temperatura: {temperatura} °C, Humitat: {humitat} %")
                    # Enviar les dades per MQTT
                    send_mqtt_data(client, temperatura, humitat)

        # Llegir dades de caigudes
        if ser_caiguda.in_waiting > 0:
            line = ser_caiguda.readline().decode('utf-8').strip()
            if line == "caiguda":
                current_time = time.time()
                # Avisar de la caiguda una sola vegada
                if current_time - last_caiguda_time >= CAIGUDA_INTERVAL:
                    print("Caiguda detectada")
                    send_mqtt_alert(client)
                    last_caiguda_time = current_time
                    
        time.sleep(1)  

except KeyboardInterrupt:
    print("Programa interromput.")

finally:
    ser_temphum.close()
    ser_caiguda.close()


